//
//  PuzzleProxy.m
//  Pseudoko
//
//  Created by David Phillip Oster on 1/8/07.
//  Copyright 2007 David Phillip Oster. 
//

#import "PuzzleProxy.h"
#import "PseudokoDocument.h"

@implementation PuzzleProxy

NSColor *myGreenColor = nil;  // color of locked cells.

+ (void)initialize{
  // tell KVC system that when isEditable changes, color changes also.
  [self setKeys:[NSArray arrayWithObject:@"isEditable"] triggerChangeNotificationsForDependentKey:@"color"];
  myGreenColor = [[NSColor colorWithDeviceRed:30./255. green:100./255. blue:35./255. alpha:1.0]retain];
}


- (id)initWithIndex:(int)indx document:(PseudokoDocument *)doc {
  if (nil != (self = [super init])) {
    index = indx;
    document = doc;
  }
  return self;
}

- (NSColor *)color {
  return [self isEditable] ? [NSColor blackColor] : myGreenColor;
}

- (int)value {
  return kValueMask & document->puzzle[index / 9][index % 9];
}

- (void)setValue:(int)n {
  int row = index / 9;
  int column = index % 9;
  document->puzzle[row][column] = (n | (document->puzzle[row][column] & ~kValueMask));
}


- (BOOL)isEditable {
  int row = index / 9;
  int column = index % 9;
  return 0 == (document->puzzle[row][column] & kIsLockedBit);
}

- (void)setIsEditable:(BOOL)editable {
  int row = index / 9;
  int column = index % 9;
  if (editable) {
    document->puzzle[row][column] &= ~kIsLockedBit;
  }else{
    document->puzzle[row][column] |= kIsLockedBit;
  }
}

@end
/*
Permission is hereby granted, free of charge, to any person obtaining
a copy of these files and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
